from typing import Callable, Tuple

from intype import has_literal, is_numeric
from texting.str_value import str_value


def duobound(
        vec: list,
        opt_x: Tuple[Callable, Callable] = (is_numeric, float),
        opt_y: Tuple[Callable, Callable] = (has_literal, str_value)
):
    size = len(vec)
    vec_x, hi_x, (filter_x, mapper_x) = [None] * size, 0, opt_x
    vec_y, hi_y, (filter_y, mapper_y) = [None] * size, 0, opt_y
    max_x = max_y = min_x = min_y = None
    for i, v in enumerate(vec):
        if filter_x(v) and (hi_x := hi_x + 1):
            vec_x[i] = v = mapper_x(v) if mapper_x else v
            if min_x is None: max_x = min_x = v
            if v > max_x: max_x = v
            elif v < min_x: min_x = v
        elif filter_y(v) and (hi_y := hi_y + 1):
            vec_y[i] = v = mapper_y(v) if mapper_y else v
            if min_y is None: max_y = min_y = v
            if v > max_y: max_y = v
            elif v < min_y: min_y = v
    return {'slice': vec_x, 'min': min_x, 'max': max_x, 'count': hi_x}, \
           {'slice': vec_y, 'min': min_y, 'max': max_y, 'count': hi_y}



from intype import is_numeric


def bound(vec, dif=False):
    hi = len(vec)
    (lo, n) = init_numeral(vec)
    ma = mi = n
    if dif and n is None: return dict(min=None, dif=None)
    while lo < (hi := hi - 1):
        if not is_numeric(x := vec[hi]): continue
        if (x := float(x)) < mi: mi = x
        elif x > ma: ma = x
    return dict(min=mi, dif=ma - mi) if dif else dict(min=mi, max=ma)


def init_numeral(vec):
    i = 0
    for i, x in enumerate(vec):
        if is_numeric(x):
            return i, float(x)
    return i + 1, None



from typing import Callable, List, Tuple

from intype import is_numeric
from veho.matrix import iso, size


def solebound(
        mx: List[list],
        opt: Tuple[Callable, Callable] = (is_numeric, float)
):
    h, w = size(mx)
    mat, hi, (filter_fn, mapper_fn) = iso(h, w, None), 0, opt
    ma = mi = None
    for i, row in enumerate(mx):
        for j, v in enumerate(row):
            if filter_fn(v) and (hi := hi + 1):
                mat[i][j] = v = mapper_fn(v) if mapper_fn else v
                if mi is None: ma = mi = v
                if v > ma: ma = v
                elif v < mi: mi = v
    return {'slice': mat, 'min': mi, 'max': ma, 'count': hi}



from typing import Callable, List, Tuple

from intype import has_literal, is_numeric
from veho.matrix import iso, size
from texting.str_value import str_value


def duobound(
        mx: List[list],
        opt_x: Tuple[Callable, Callable] = (is_numeric, float),
        opt_y: Tuple[Callable, Callable] = (has_literal, str_value)
):
    h, w = size(mx)
    mx_x, hi_x, (filter_x, mapper_x) = iso(h, w, None), 0, opt_x
    mx_y, hi_y, (filter_y, mapper_y) = iso(h, w, None), 0, opt_y
    max_x = max_y = min_x = min_y = None
    for i, row in enumerate(mx):
        for j, v in enumerate(row):
            if filter_x(v) and (hi_x := hi_x + 1):
                mx_x[i][j] = v = mapper_x(v) if mapper_x else v
                if min_x is None: max_x = min_x = v
                if v > max_x: max_x = v
                elif v < min_x: min_x = v
            elif filter_y(v) and (hi_y := hi_y + 1):
                mx_y[i][j] = v = mapper_y(v) if mapper_y else v
                if min_y is None: max_y = min_y = v
                if v > max_y: max_y = v
                elif v < min_y: min_y = v
    return {'slice': mx_x, 'min': min_x, 'max': max_x, 'count': hi_x}, \
           {'slice': mx_y, 'min': min_y, 'max': max_y, 'count': hi_y}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aryth',
 'aryth.bound_matrix',
 'aryth.bound_vector',
 'aryth.enum_bound_keys',
 'aryth.math',
 'aryth.stat']

package_data = \
{'': ['*']}

install_requires = \
['intype>=0.0.2,<0.0.3', 'texting>=0.0.2,<0.0.3', 'veho>=0.0.2,<0.0.3']

setup_kwargs = {
    'name': 'aryth',
    'version': '0.0.1',
    'description': 'numerical tools',
    'long_description': '',
    'author': 'Hoyeung Wong',
    'author_email': 'hoyeungw@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hoyeungw/aryth',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Tuple


def count(arr: list):
    return len(arr) if arr else 0


def average(arr: list):
    return sum(arr) / len(arr) if arr else 0


def bound(arr: list) -> Tuple[int or float, int or float]:
    """
    :return Tuple[max, min]
    """
    if not arr: return None, None
    peak = vale = arr[0]
    for x in arr:
        if x > peak: peak = x
        if x < vale: vale = x
    return peak, vale



from intype import is_numeric
from veho.matrix import size


def bound(mx, dif=False):
    h, w = size(mx)
    (i, j, n) = init_numeral(mx)
    ma = mi = n
    if n is None or h is None:
        return dict(min=mi, dif=None) if dif else dict(min=mi, max=ma)
    while i < h:
        row = mx[i]
        while j < w:
            if is_numeric(x := row[j]):
                if (x := float(x)) < mi: mi = x
                elif x > ma: ma = x
            j += 1
        j = 0
        i += 1
    return dict(min=mi, dif=ma - mi) if dif else dict(min=mi, max=ma)


def init_numeral(mx):
    i = j = 0
    for i, r in enumerate(mx):
        for j, x in enumerate(r):
            if is_numeric(x):
                return i, j, float(x)
    return i + 1, j + 1, None



from typing import Callable, Tuple

from intype import is_numeric


def solebound(
        vec: list,
        opt: Tuple[Callable, Callable or None] = (is_numeric, float)
):
    size = len(vec)
    ve, hi, (filter_fn, mapper) = [None] * size, 0, opt
    ma = mi = None
    for i, v in enumerate(vec):
        if filter_fn(v) and (hi := hi + 1):
            ve[i] = v = mapper(v) if mapper else v
            if not ma and not mi: ma = mi = v
            if v > ma:
                ma = v
            elif v < mi:
                mi = v
    return {'slice': ve, 'min': mi, 'max': ma, 'count': hi}
